@@ -29,9 +29,13 @@
 
 #include "rest/rest-private.h"
 #include "youtube-proxy.h"
-#include "youtube-proxy-private.h"
 
-G_DEFINE_TYPE (YoutubeProxy, youtube_proxy, REST_TYPE_PROXY)
+typedef struct {
+  char *developer_key;
+  char *user_auth;
+} YoutubeProxyPrivate;
+
+G_DEFINE_TYPE_WITH_PRIVATE (YoutubeProxy, youtube_proxy, REST_TYPE_PROXY)
 
 #define UPLOAD_URL \
   "http://uploads.gdata.youtube.com/feeds/api/users/default/uploads"
@@ -40,8 +44,11 @@ enum {
   PROP_0,
   PROP_DEVELOPER_KEY,
   PROP_USER_AUTH,
+  N_PROPS,
 };
 
+static GParamSpec *properties [N_PROPS];
+
 GQuark
 youtube_proxy_error_quark (void)
 {
@@ -49,10 +56,13 @@ youtube_proxy_error_quark (void)
 }
 
 static void
-youtube_proxy_get_property (GObject *object, guint property_id,
-                              GValue *value, GParamSpec *pspec)
+youtube_proxy_get_property (GObject    *object,
+                            guint       property_id,
+                            GValue     *value,
+                            GParamSpec *pspec)
 {
-  YoutubeProxyPrivate *priv = YOUTUBE_PROXY_GET_PRIVATE (object);
+  YoutubeProxy *self = YOUTUBE_PROXY (object);
+  YoutubeProxyPrivate *priv = youtube_proxy_get_instance_private (self);
 
   switch (property_id) {
   case PROP_DEVELOPER_KEY:
@@ -67,10 +77,13 @@ youtube_proxy_get_property (GObject *object, guint property_id,
 }
 
 static void
-youtube_proxy_set_property (GObject *object, guint property_id,
-                           const GValue *value, GParamSpec *pspec)
+youtube_proxy_set_property (GObject      *object,
+                            guint         property_id,
+                            const GValue *value,
+                            GParamSpec   *pspec)
 {
-  YoutubeProxyPrivate *priv = YOUTUBE_PROXY_GET_PRIVATE (object);
+  YoutubeProxy *self = YOUTUBE_PROXY (object);
+  YoutubeProxyPrivate *priv = youtube_proxy_get_instance_private (self);
 
   switch (property_id) {
   case PROP_DEVELOPER_KEY:
@@ -89,7 +102,8 @@ youtube_proxy_set_property (GObject *object, guint property_id,
 static void
 youtube_proxy_finalize (GObject *object)
 {
-  YoutubeProxyPrivate *priv = YOUTUBE_PROXY_GET_PRIVATE (object);
+  YoutubeProxy *self = YOUTUBE_PROXY (object);
+  YoutubeProxyPrivate *priv = youtube_proxy_get_instance_private (self);
 
   g_free (priv->developer_key);
   g_free (priv->user_auth);
@@ -101,36 +115,34 @@ static void
 youtube_proxy_class_init (YoutubeProxyClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
-
-  g_type_class_add_private (klass, sizeof (YoutubeProxyPrivate));
 
   object_class->get_property = youtube_proxy_get_property;
   object_class->set_property = youtube_proxy_set_property;
   object_class->finalize = youtube_proxy_finalize;
 
-  pspec = g_param_spec_string ("developer-key",  "developer-key",
-                               "The developer API key", NULL,
-                               G_PARAM_READWRITE|
-                               G_PARAM_CONSTRUCT_ONLY|
-                               G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (object_class,
-                                   PROP_DEVELOPER_KEY,
-                                   pspec);
+  properties [PROP_DEVELOPER_KEY] =
+    g_param_spec_string ("developer-key",
+                         "developer-key",
+                         "The developer API key",
+                         NULL,
+                         (G_PARAM_READWRITE |
+                          G_PARAM_CONSTRUCT_ONLY |
+                          G_PARAM_STATIC_STRINGS));
 
-  pspec = g_param_spec_string ("user-auth",  "user-auth",
-                               "The ClientLogin token", NULL,
-                               G_PARAM_READWRITE|
-                               G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (object_class,
-                                   PROP_USER_AUTH,
-                                   pspec);
+  properties [PROP_USER_AUTH] =
+    g_param_spec_string ("user-auth",
+                         "user-auth",
+                         "The ClientLogin token",
+                         NULL,
+                         (G_PARAM_READWRITE |
+                          G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
 static void
 youtube_proxy_init (YoutubeProxy *self)
 {
-  self->priv = YOUTUBE_PROXY_GET_PRIVATE (self);
 }
 
 RestProxy *
@@ -151,16 +163,17 @@ youtube_proxy_new_with_auth (const char *developer_key,
 }
 
 void
-youtube_proxy_set_user_auth (YoutubeProxy *proxy,
+youtube_proxy_set_user_auth (YoutubeProxy *self,
                              const gchar  *user_auth)
 {
-  YoutubeProxyPrivate *priv = proxy->priv;
+  YoutubeProxyPrivate *priv = youtube_proxy_get_instance_private (self);
 
   priv->user_auth = g_strdup (user_auth);
 }
 
 static gchar *
-_construct_upload_atom_xml (GHashTable *fields, gboolean incomplete)
+_construct_upload_atom_xml (GHashTable *fields,
+                            gboolean    incomplete)
 {
   GHashTableIter iter;
   gpointer key, value;
@@ -215,7 +228,7 @@ _set_upload_headers (YoutubeProxy *self,
                      SoupMessageHeaders *headers,
                      const gchar *filename)
 {
-  YoutubeProxyPrivate *priv = self->priv;
+  YoutubeProxyPrivate *priv = youtube_proxy_get_instance_private (self);
   gchar *user_auth_header;
   gchar *devkey_header;
   gchar *basename;
@@ -246,6 +259,9 @@ typedef struct {
   GObject *weak_object;
   gpointer user_data;
   gsize uploaded;
+#ifndef WITH_SOUP_2
+  GCancellable *cancellable;
+#endif
 } YoutubeProxyUploadClosure;
 
 static void
@@ -255,7 +271,11 @@ _upload_async_weak_notify_cb (gpointer *data,
   YoutubeProxyUploadClosure *closure =
     (YoutubeProxyUploadClosure *) data;
 
+#ifdef WITH_SOUP_2
   _rest_proxy_cancel_message (REST_PROXY (closure->proxy), closure->message);
+#else
+  g_cancellable_cancel (closure->cancellable);
+#endif
 }
 
 static void
@@ -267,6 +287,9 @@ _upload_async_closure_free (YoutubeProxyUploadClosure *closure)
                          closure);
 
   g_object_unref (closure->proxy);
+#ifndef WITH_SOUP_2
+  g_object_unref (closure->cancellable);
+#endif
 
   g_slice_free (YoutubeProxyUploadClosure, closure);
 }
@@ -286,6 +309,9 @@ _upload_async_closure_new (YoutubeProxy *self,
   closure->message = message;
   closure->weak_object = weak_object;
   closure->user_data = user_data;
+#ifndef WITH_SOUP_2
+  closure->cancellable = g_cancellable_new ();
+#endif
 
   if (weak_object != NULL)
     g_object_weak_ref (weak_object,
@@ -295,41 +321,67 @@ _upload_async_closure_new (YoutubeProxy *self,
 }
 
 static void
-_upload_completed_cb (SoupSession *session,
-                      SoupMessage *message,
+_upload_completed_cb (SoupMessage *message,
+                      GBytes      *payload,
+                      GError      *error,
                       gpointer     user_data)
 {
   YoutubeProxyUploadClosure *closure =
     (YoutubeProxyUploadClosure *) user_data;
-  GError *error = NULL;
+  gsize length;
+  gconstpointer data;
+  guint status_code;
+  const char *reason_phrase;
 
   if (closure->callback == NULL)
     return;
 
-  if (message->status_code < 200 || message->status_code >= 300)
-    error = g_error_new_literal (REST_PROXY_ERROR,
-                                 message->status_code,
-                                 message->reason_phrase);
+#ifdef WITH_SOUP_2
+  status_code = message->status_code;
+  reason_phrase = message->reason_phrase;
+#else
+  status_code = soup_message_get_status (message);
+  reason_phrase = soup_message_get_reason_phrase (message);
+#endif
 
-  closure->callback (closure->proxy, message->response_body->data,
-                     message->request_body->length,
-                     message->request_body->length,
+  if (status_code < 200 || status_code >= 300)
+    {
+      g_clear_error (&error);
+      error = g_error_new_literal (REST_PROXY_ERROR,
+                                   status_code,
+                                   reason_phrase);
+    }
+
+  data = g_bytes_get_data (payload, &length);
+  closure->callback (closure->proxy, data, length, length,
                      error, closure->weak_object, closure->user_data);
+  g_bytes_unref (payload);
 
   _upload_async_closure_free (closure);
 }
 
 static void
 _message_wrote_data_cb (SoupMessage               *msg,
+#ifdef WITH_SOUP_2
                         SoupBuffer                *chunk,
+#else
+                        gsize                      chunk_size,
+#endif
                         YoutubeProxyUploadClosure *closure)
 {
-  closure->uploaded = closure->uploaded + chunk->length;
+#ifdef WITH_SOUP_2
+  gsize chunk_size = chunk->length;
+  goffset content_length = msg->request_body->length;
+#else
+  goffset content_length = soup_message_headers_get_content_length (soup_message_get_request_headers (msg));
+#endif
 
-  if (closure->uploaded < msg->request_body->length)
+  closure->uploaded = closure->uploaded + chunk_size;
+
+  if (closure->uploaded < content_length)
     closure->callback (closure->proxy,
                        NULL,
-                       msg->request_body->length,
+                       content_length,
                        closure->uploaded,
                        NULL,
                        closure->weak_object,
@@ -364,7 +416,12 @@ youtube_proxy_upload_async (YoutubeProxy              *self,
   SoupMultipart *mp;
   SoupMessage *message;
   SoupMessageHeaders *part_headers;
+  SoupMessageHeaders *request_headers;
+#ifdef WITH_SOUP_2
   SoupBuffer *sb;
+#else
+  GBytes *sb;
+#endif
   gchar *content_type;
   gchar *atom_xml;
   GMappedFile *map;
@@ -380,10 +437,17 @@ youtube_proxy_upload_async (YoutubeProxy              *self,
 
   atom_xml = _construct_upload_atom_xml (fields, incomplete);
 
+#ifdef WITH_SOUP_2
   sb = soup_buffer_new_with_owner (atom_xml,
                                    strlen(atom_xml),
                                    atom_xml,
                                    (GDestroyNotify) g_free);
+#else
+  sb = g_bytes_new_with_free_func (atom_xml,
+                                   strlen (atom_xml),
+                                   (GDestroyNotify) g_free,
+                                   atom_xml);
+#endif
 
   part_headers = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
 
@@ -393,7 +457,11 @@ youtube_proxy_upload_async (YoutubeProxy              *self,
 
   soup_multipart_append_part (mp, part_headers, sb);
 
+#ifdef WITH_SOUP_2
   soup_buffer_free (sb);
+#else
+  g_bytes_unref (sb);
+#endif
 
   content_type = g_content_type_guess (
       filename,
@@ -401,24 +469,37 @@ youtube_proxy_upload_async (YoutubeProxy              *self,
       g_mapped_file_get_length (map),
       NULL);
 
+#ifdef WITH_SOUP_2
   sb = soup_buffer_new_with_owner (g_mapped_file_get_contents (map),
                                    g_mapped_file_get_length (map),
                                    map,
                                    (GDestroyNotify) g_mapped_file_unref);
+#else
+  sb = g_bytes_new_with_free_func (g_mapped_file_get_contents (map),
+                                   g_mapped_file_get_length (map),
+                                   (GDestroyNotify) g_mapped_file_unref,
+                                   map);
+#endif
 
   soup_message_headers_replace (part_headers, "Content-Type", content_type);
 
   soup_multipart_append_part (mp, part_headers, sb);
 
+#ifdef WITH_SOUP_2
   soup_buffer_free (sb);
-
   soup_message_headers_free (part_headers);
-
   message = soup_form_request_new_from_multipart (UPLOAD_URL, mp);
+  request_headers = message->request_headers;
+#else
+  g_bytes_unref (sb);
+  soup_message_headers_unref (part_headers);
+  message = soup_message_new_from_multipart (UPLOAD_URL, mp);
+  request_headers = soup_message_get_request_headers (message);
+#endif
 
   soup_multipart_free (mp);
 
-  _set_upload_headers (self, message->request_headers, filename);
+  _set_upload_headers (self, request_headers, filename);
 
   closure = _upload_async_closure_new (self, callback, message, weak_object,
                                        user_data);
@@ -429,7 +510,13 @@ youtube_proxy_upload_async (YoutubeProxy              *self,
                     closure);
 
 
-  _rest_proxy_queue_message (REST_PROXY (self), message, _upload_completed_cb,
+  _rest_proxy_queue_message (REST_PROXY (self), message,
+#ifdef WITH_SOUP_2
+                             NULL,
+#else
+                             closure->cancellable,
+#endif
+                             _upload_completed_cb,
                              closure);
 
   return TRUE;
